import base64
import os

from goodconf import GoodConf, Value

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "saltdash.settings")


class Config(GoodConf):
    """Configuration for saltdash"""

    DEBUG = Value(default=False, help="Enable debugging.")
    SECRET_KEY = Value(
        initial=lambda: base64.b64encode(os.urandom(60)).decode(),
        help="a long random string you keep secret "
        "https://docs.djangoproject.com/en/2.2/ref/settings/#secret-key",
    )
    DATABASE_URL = Value(default="postgres://localhost:5432/salt")
    ALLOWED_HOSTS = Value(
        default=["*"],
        help="Hosts allowed to serve the "
        "site "
        "https://docs.djangoproject.com/en/2.2/ref/settings/#allowed-hosts",
    )
    HIDE_OUTPUT = Value(
        default=["pillar.*"],
        help="List of modules to hide the output from in the web interface.",
    )
    GITHUB_TEAM_ID = Value(default="")
    GITHUB_CLIENT_ID = Value(default="")
    GITHUB_CLIENT_SECRET = Value(default="")
    SENTRY_DSN = Value(default="")
    LISTEN = Value(default="127.0.0.1:8077", help="Socket for webserver to listen on.")


config = Config(
    file_env_var="SALTDASH_CONF",
    default_files=["saltdash.yml", "/etc/saltdash/saltdash.yml"],
)


def generate_config():
    """Entrypoint for dumping out sample config"""
    print(config.generate_yaml())



import sys

from django.conf import settings
from django.core.wsgi import get_wsgi_application

application = get_wsgi_application()

# Warm-up wsgi app on import
warmup = application(
    {
        "REQUEST_METHOD": "GET",
        "SERVER_NAME": "127.0.0.1",
        "SERVER_PORT": 80,
        "PATH_INFO": "/-/alive/",
        "wsgi.input": sys.stdin,
    },
    lambda x, y: None,
)



import json
from urllib.parse import urlencode

from django.template import Library
from django.utils.safestring import mark_safe

from ..models import Result

register = Library()


@register.filter
def pretty_json(dict_val):
    return json.dumps(dict_val, indent=2)


@register.simple_tag
def get_minions():
    return Result.objects.order_by("minion").values_list("minion", flat=True).distinct()


@register.simple_tag(takes_context=True)
def url_replace(context, **kwargs):
    """Replace a single GET parameter while maintaining others"""
    query = context["request"].GET.dict()
    query.update(kwargs)
    return urlencode(query)


@register.filter
def pretty_time(ms):
    """Convert milliseconds to more readable time"""
    if ms > 60000:
        min = int(ms / 60000)
        ms = ms - min * 60000
        return "{}:{:05.2f} min".format(min, ms / 1000)
    if ms > 1000:
        return "{:.2f} sec".format(ms / 1000)
    # pad one character right to keep alignment in states
    return mark_safe("{:.0f} ms&nbsp;".format(round(ms, 0)))


@register.filter
def clipped_page_range(page_range: range, current_page: int) -> list:
    """Abbreviated page range to avoid a massive pagination"""
    page_range = list(page_range)
    if len(page_range) <= 8:
        return page_range
    # show first two and last two pages
    start = page_range[:2]
    end = page_range[-2:]

    # prevent middle from going beyond start/end
    if current_page >= start[0] + 2:
        first_middle = current_page - 2
    else:
        first_middle = start[0]
    if current_page <= end[-1] - 2:
        last_middle = current_page + 2
    else:
        last_middle = end[-1]
    middle = list(range(first_middle, last_middle + 1))

    # add gaps in pages as None if necessary
    if middle[0] > start[-1] + 1:
        pages = start + [None] + middle
    else:
        pages = list(set(start + middle))
    if middle[-1] + 1 > end[0]:
        for p in end:
            if pages[-1] < p:
                pages.append(p)
    else:
        pages = pages + [None] + end
    return pages



from decimal import Decimal
from unittest import TestCase

from saltdash.dash.templatetags.dash_tags import clipped_page_range, pretty_time


class PrettyTimeTestCase(TestCase):
    def test_minutes(self):
        self.assertEqual(pretty_time(69305.23), "1:09.31 min")

    def test_seconds(self):
        self.assertEqual(pretty_time(20305.23), "20.31 sec")

    def test_milliseconds(self):
        self.assertEqual(pretty_time(305.23), "305 ms&nbsp;")


class ClippedPageRangeTestCase(TestCase):
    def test_long(self):
        clipped = clipped_page_range(list(range(1, 21)), 10)
        self.assertListEqual(clipped, [1, 2, None, 8, 9, 10, 11, 12, None, 19, 20])

    def test_short(self):
        clipped = clipped_page_range(list(range(1, 8)), 5)
        self.assertListEqual(clipped, [1, 2, 3, 4, 5, 6, 7])

    def test_left(self):
        clipped = clipped_page_range(list(range(1, 21)), 2)
        self.assertListEqual(clipped, [1, 2, 3, 4, None, 19, 20])

    def test_right(self):
        clipped = clipped_page_range(list(range(1, 21)), 18)
        self.assertListEqual(clipped, [1, 2, None, 16, 17, 18, 19, 20])



import logging
from re import compile

from django.conf import settings
from django.http import HttpResponseRedirect
from django.utils.http import is_safe_url

log = logging.getLogger(__name__)

EXEMPT_URLS = [compile(settings.LOGIN_URL.lstrip("/"))] + [
    compile(expr) for expr in getattr(settings, "LOGIN_EXEMPT_URLS", [])
]

# https://gist.github.com/agusmakmun/b71ac536124e0535a8b076989f8cfcd3
class LoginRequiredMiddleware:
    """
    Middleware that requires a user to be authenticated to view any page other
    than LOGIN_URL. Exemptions to this requirement can optionally be specified
    in settings via a list of regular expressions in LOGIN_EXEMPT_URLS (which
    you can copy from your urls.py).
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        # Before the view is called
        if not request.user.is_authenticated:
            path = request.path_info.lstrip("/")
            if not any(m.match(path) for m in EXEMPT_URLS):
                redirect_to = settings.LOGIN_URL
                # Add 'next' GET variable to support redirection after login
                if len(path) > 0 and is_safe_url(
                    url=request.path_info, host=request.get_host()
                ):
                    redirect_to = "{}?next={}".format(
                        settings.LOGIN_URL, request.path_info
                    )
                return HttpResponseRedirect(redirect_to)
        response = self.get_response(request)
        # After the view is called
        return response



from django.urls import path, reverse_lazy
from django.views.generic import RedirectView

from . import views

app_name = "dash"

urlpatterns = [
    # placeholder for dashboard
    path("", RedirectView.as_view(url=reverse_lazy("dash:job_list"))),
    path("get-started/", views.get_started, name="get_started"),
    path("job/", views.job_list, name="job_list"),
    path("job/<int:jid>/", views.job_detail, name="job_detail"),
    path(
        "job/<int:jid>/success/",
        views.job_detail,
        {"success": True},
        name="job_detail_success",
    ),
    path(
        "job/<int:jid>/failure/",
        views.job_detail,
        {"success": False},
        name="job_detail_failed",
    ),
    path("job/<int:jid>/<str:minion>/", views.result_detail, name="result_detail"),
    path("result/", views.result_list, name="result_list"),
]



import datetime

from django.core.management.base import BaseCommand, CommandError
from django.db.models import F, Func, Value
from django.utils import timezone as tz
from saltdash.dash import models


class Command(BaseCommand):
    help = "Removes items from job cache database older than X days"

    def add_arguments(self, parser):
        parser.add_argument(
            "days", type=int, help="Delete job cache items older than this many days"
        )
        parser.add_argument(
            "--no-input",
            action="store_true",
            dest="no_input",
            help="Don't ask for confirmation prior to deleting items",
        )

    def handle(self, *args, **options):
        days = options["days"]
        if days < 1:
            raise CommandError(
                "Not purging because less than 1 day specified. "
                "Days = {}".format(days)
            )
        before = tz.now() - datetime.timedelta(days=days)

        results = models.Result.objects.filter(completed__lt=before)
        jobs = models.Job.objects.annotate(
            date=Func(F("jid"), Value("YYYYMMDDHH24MISSMS"), function="TO_TIMESTAMP")
        ).filter(date__lt=before)
        self.stdout.write(
            self.style.WARNING(
                "Deleting {} jobs and {} results created before {}.".format(
                    jobs.count(), results.count(), before
                )
            )
        )

        if not options["no_input"]:
            response = input("Are you sure you want to delete these items? [y/N]")
            if response.lower() not in ["y", "yes"]:
                self.stdout.write("Nothing deleted.")
                return
        results.delete()
        jobs.delete()
        self.stdout.write("Job cache purged.")



from django.contrib import admin
from django.contrib.admin.decorators import register

from . import models


class SuccessFilter(admin.SimpleListFilter):
    title = "success"
    parameter_name = "success"

    def lookups(self, request, model_admin):
        return (("1", "Yes"), ("0", "No"))

    def queryset(self, request, queryset):
        if self.value() == "1":
            return queryset.successes()
        if self.value() == "0":
            return queryset.failures()


@register(models.Result)
class ResultAdmin(admin.ModelAdmin):
    list_filter = [SuccessFilter, "fun", "minion"]
    list_display = ["jid", "fun", "minion", "completed", "success_method"]

    def success_method(self, obj):
        return obj.was_success

    success_method.boolean = True
    success_method.short_description = "success"


@register(models.Job)
class JobAdmin(admin.ModelAdmin):
    pass



import os

from django.utils.log import DEFAULT_LOGGING

LOGLEVEL = os.getenv("LOGLEVEL", "info").upper()


def _log_format():
    keys = [
        "asctime",
        "created",
        "levelname",
        "levelno",
        "filename",
        "funcName",
        "lineno",
        "module",
        "message",
        "name",
        "pathname",
        "process",
        "processName",
    ]
    return " ".join(["%({0:s})".format(i) for i in keys])


LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "default": {
            "format": _log_format(),
            "class": "pythonjsonlogger.jsonlogger.JsonFormatter",
        }
    },
    "handlers": {
        # console logs to stderr
        "console": {"class": "logging.StreamHandler", "formatter": "default"},
    },
    "loggers": {
        # default for all undefined Python modules
        "": {"level": "WARNING", "handlers": ["console"]},
        # Our application code
        "saltdash": {
            "level": LOGLEVEL,
            "handlers": ["console"],
            # Avoid double logging because of root logger
            "propagate": False,
        },
    },
}
if "SYSLOG_IDENTIFIER" in os.environ:
    try:
        from systemd import journal

        LOGGING["formatters"]["default"] = {"format": "%(message)s"}
        LOGGING["handlers"]["console"] = {
            "class": "systemd.journal.JournalHandler",
            "SYSLOG_IDENTIFIER": os.getenv("SYSLOG_IDENTIFIER"),
        }
    except ImportError:
        pass

# Add runserver request logging back in
for k in ["formatters", "handlers", "loggers"]:
    LOGGING[k]["django.server"] = DEFAULT_LOGGING[k]["django.server"]
try:
    # setup pretty logging for local dev
    import readable_log_formatter

    LOGGING["formatters"]["default"] = {
        "()": "readable_log_formatter.ReadableFormatter"
    }
except ImportError:
    pass



from django.conf import settings
from django.contrib import admin
from django.urls import include, path

urlpatterns = [
    path("-/", include("django_alive.urls")),
    path("admin/", admin.site.urls),
    path(
        "{}/".format(settings.SOCIAL_AUTH_URL_PREFIX),
        include("social_django.urls", namespace="social"),
    ),
    path("", include("saltdash.dash.urls")),
]



"""
Django settings for saltdash project.

Generated by 'django-admin startproject' using Django 2.0.1.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import os
import logging.config
from pathlib import Path

import dj_database_url
from django.urls import reverse_lazy

from saltdash import config

config.load()

if config.SENTRY_DSN:
    import sentry_sdk
    from sentry_sdk.integrations.django import DjangoIntegration
    sentry_sdk.init(config.SENTRY_DSN, integrations=[DjangoIntegration()])

from ._logging import LOGGING

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = (Path(os.path.abspath(__file__)) / ".." / "..").resolve()

# Disable Django's logging setup
LOGGING_CONFIG = None
logging.config.dictConfig(LOGGING)
log = logging.getLogger(__name__)

if config.config_file:
    log.info("Config loaded from %s.", config.config_file)
else:
    log.info("Config loaded from environment.")

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = config.SECRET_KEY

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = config.DEBUG

ALLOWED_HOSTS = config.ALLOWED_HOSTS
SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")
SECURE_REDIRECT_EXEMPT = [r"^-/"]  # django-alive URLs
ALIVE_CHECKS = {
    "django_alive.checks.check_database": {},
    "django_alive.checks.check_staticfile": {
        "filename": "app.css",
    },
}


# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "social_django",
    "saltdash.dash",
]

MIDDLEWARE = [
    "django_alive.middleware.healthcheck_bypass_host_check",
    "django.middleware.security.SecurityMiddleware",
    "whitenoise.middleware.WhiteNoiseMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "saltdash.core.middleware.LoginRequiredMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "saltdash.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [str(BASE_DIR / "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

WSGI_APPLICATION = "saltdash.wsgi.application"
TEST_RUNNER = "saltdash.core.PytestTestRunner"

# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

DATABASES = {"default": dj_database_url.parse(config.DATABASE_URL)}


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
    },
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = "/static/"
STATICFILES_DIRS = [str(BASE_DIR / ".." / "client" / "dist")]
STATIC_ROOT = str(BASE_DIR / "static")
STATICFILES_STORAGE = "whitenoise.storage.CompressedManifestStaticFilesStorage"

# Python Social Auth
AUTHENTICATION_BACKENDS = [
    "social_core.backends.github.GithubTeamOAuth2",
    "django.contrib.auth.backends.ModelBackend",
]

SOCIAL_AUTH_URL_PREFIX = "auth"
# Bypass LoginRequiredMiddleware for social auth login and callback
LOGIN_EXEMPT_URLS = ["{}/.*".format(SOCIAL_AUTH_URL_PREFIX)]
LOGIN_URL = reverse_lazy("social:begin", args=["github-team"])
LOGIN_REDIRECT_URL = "/"


# create token https://github.com/settings/tokens
# curl -H "Authorization: token <token>" \
#       https://api.github.com/orgs/<org>/teams
SOCIAL_AUTH_GITHUB_TEAM_ID = config.GITHUB_TEAM_ID

# https://github.com/organizations/<org>/settings/applications
SOCIAL_AUTH_GITHUB_TEAM_KEY = config.GITHUB_CLIENT_ID
SOCIAL_AUTH_GITHUB_TEAM_SECRET = config.GITHUB_CLIENT_SECRET
# Need to read teams to know if user can login
SOCIAL_AUTH_GITHUB_TEAM_SCOPE = ["read:org"]

if (
    not SOCIAL_AUTH_GITHUB_TEAM_ID
    or not SOCIAL_AUTH_GITHUB_TEAM_KEY
    or not SOCIAL_AUTH_GITHUB_TEAM_SECRET
):
    log.warning(
        "GitHub login environment variables not present. "
        "Turning off login requirement."
    )
    # If Github is not setup, don't require login
    MIDDLEWARE.remove("saltdash.core.middleware.LoginRequiredMiddleware")

HIDE_OUTPUT = config.HIDE_OUTPUT



import fnmatch
import json

from django.conf import settings
from django.contrib.postgres.fields import ArrayField, JSONField
from django.db import models
from django.db.models import Q
from django.urls import reverse
from django.utils.functional import cached_property


class Job(models.Model):
    jid = models.CharField(verbose_name="job id", max_length=50, primary_key=True)
    load = JSONField()

    class Meta:
        db_table = "jids"
        ordering = ["-jid"]

    @cached_property
    def user(self):
        return self.load.get("user", None)

    @cached_property
    def args(self):
        return [a for a in self.load["arg"] if isinstance(a, str)]

    @cached_property
    def kwargs(self):
        for a in self.load["arg"]:
            if isinstance(a, dict) and a.get("__kwarg__"):
                del (a["__kwarg__"])
                return a

    @property
    def saltreturn_set(self):
        # Fake foreign key relation
        return Result.objects.filter(jid=self.jid)

    @cached_property
    def successes(self):
        return self.saltreturn_set.successes().count()

    @cached_property
    def failures(self):
        return self.saltreturn_set.failures().count()

    @cached_property
    def completed(self):
        try:
            return self.saltreturn_set.all().order_by("-completed")[0].completed
        except IndexError:
            return None

    @property
    def targets(self) -> list:
        tgt = self.load["tgt"]
        if self.load["tgt_type"] == "list" and not isinstance(tgt, str):
            return tgt
        return [tgt]

    def __str__(self) -> str:
        fun = self.load["fun"]
        target_count = len(self.targets)
        if target_count > 10:
            targets = "{} targets".format(target_count)
        else:
            targets = ", ".join(self.targets)
        val = "{}: salt '{}' {}".format(self.jid, targets, fun)
        if self.user:
            val += " by {}".format(self.user)
        return val


class ResultQuerySet(models.QuerySet):
    def successes(self):
        return self.filter(
            Q(full_ret__contains={"retcode": 0})
            | Q(return_val__contains={"success": True})
        )

    def failures(self):
        return self.exclude(
            Q(full_ret__contains={"retcode": 0})
            | Q(return_val__contains={"success": True})
        )


class Result(models.Model):
    auto_id = models.AutoField(primary_key=True)
    minion = models.CharField(max_length=100, db_index=True, db_column="id")
    # This can't be a foreign key due to the order Salt might insert data into
    # the tables.
    jid = models.CharField(verbose_name="job id", max_length=50, db_index=True)
    fun = models.CharField("function", max_length=100)
    fun_args = ArrayField(
        models.CharField(max_length=255), verbose_name="function args", null=True
    )
    completed = models.DateTimeField(db_column="alter_time")
    full_ret = JSONField("full result")
    return_val = JSONField("result", db_column="return")
    success = models.BooleanField()

    objects = ResultQuerySet.as_manager()

    class Meta:
        db_table = "salt_returns"
        ordering = ["-completed"]

    def __str__(self):
        return "{}: {} on {}".format(self.jid, self.fun, self.minion)

    def get_absolute_url(self):
        return reverse("dash:result_detail", args=[self.jid, self.minion])

    @property
    def was_success(self):
        if "retcode" in self.full_ret:
            return self.full_ret["retcode"] == 0
        # Master results don't use retcode
        try:
            return self.return_val.get("success", False)
        except AttributeError:
            return False

    @cached_property
    def job(self) -> Job:
        try:
            return Job.objects.get(jid=self.jid)
        except Job.DoesNotExist:
            return Job(
                jid=self.full_ret["jid"],
                load={
                    "arg": self.full_ret.get("fun_args", []),
                    "fun": self.full_ret["fun"],
                    "jid": self.full_ret["jid"],
                    "tgt": [self.full_ret["id"]],
                    "user": self.full_ret["user"],
                    "tgt_type": "list",
                },
            )

    @cached_property
    def return_val_as_json(self) -> str:
        return json.dumps(self.return_val, indent=2)

    @property
    def result_type(self) -> str:
        for hidden in settings.HIDE_OUTPUT:
            if fnmatch.fnmatch(self.full_ret.get("fun", ""), hidden):
                return "hidden"
        if isinstance(self.return_val, dict):
            if self.return_val.get("fun") == "runner.state.orchestrate":
                return "orchestrate"
            try:
                list(self.return_val.values())[0]["__run_num__"]
                return "state"
            except (TypeError, IndexError, KeyError):
                return "json"
        return "text"

    @property
    def has_states(self) -> bool:
        return self.result_type in ["state", "orchestrate"]

    def states_with_status(self, status: str) -> int:
        return len([s for s in self.states if s["status"] == status])

    @cached_property
    def states_changed(self) -> int:
        return self.states_with_status("changed")

    @cached_property
    def states_failed(self) -> int:
        return self.states_with_status("failed")

    @cached_property
    def states_failed_requisite(self) -> int:
        return self.states_with_status("requisite-failed")

    @cached_property
    def states_unchanged(self) -> int:
        return self.states_with_status("unchanged")

    @cached_property
    def states(self) -> list:
        if self.result_type == "state":
            state_dict = self.return_val
        elif self.result_type == "orchestrate":
            state_dict = list(self.return_val["return"]["data"].values())[0]
        else:
            raise TypeError("Return must be for a state")
        sorted_run = sorted(state_dict.items(), key=lambda s: s[1]["__run_num__"])
        return [_convert_state(*args) for args in sorted_run]

    @cached_property
    def duration(self) -> float:
        return sum([s["duration"] for s in self.states])


def _convert_state(key: str, data: dict) -> dict:
    """Make state data more usable"""
    mod, id, name, fun = key.split("_|-")
    state = {
        "module": mod,
        "id": id,
        "function": fun,
        "name": name,
        "success": data["result"],
        "changed": bool(data["changes"]),
        "sls": data.get("__sls__"),
        "changes": data["changes"],
        "duration": data.get("duration", 0),
        "start_time": data.get("start_time"),
        "comment": data.get("comment", ""),
        "order": data["__run_num__"],
        "jid": data.get("__jid__"),
    }
    if not state["success"]:
        if state["comment"].startswith("One or more requisite failed:"):
            state["status"] = "requisite-failed"
        else:
            state["status"] = "failed"
    elif state["changed"]:
        state["status"] = "changed"
    else:
        state["status"] = "unchanged"
    return state



from django.conf import settings
from django.core.paginator import InvalidPage, Page, Paginator
from django.db.models import QuerySet
from django.http import (
    Http404,
    HttpRequest,
    HttpResponsePermanentRedirect,
    HttpResponseRedirect,
)
from django.shortcuts import get_object_or_404, render
from django.urls import reverse

from .models import Job, Result

ITEMS_PER_PAGE = 25


def paginate_queryset(queryset: QuerySet, request: HttpRequest) -> dict:
    """Paginate the queryset, if needed."""
    paginator = Paginator(queryset, ITEMS_PER_PAGE)
    page = request.GET.get("page") or 1
    try:
        page_number = int(page)
    except ValueError:
        if page == "last":
            page_number = paginator.num_pages
        else:
            raise Http404("Page is not 'last', nor can it be converted to an int.")
    try:
        page = paginator.page(page_number)
        return {
            "paginator": paginator,
            "page_obj": page,
            "object_list": page.object_list,
            "is_paginated": page.has_other_pages(),
        }
    except InvalidPage as e:
        raise Http404(
            "Invalid page ({page_number}): {exp}".format(
                page_number=page_number, exp=str(e)
            )
        )


def get_started(request):
    template = "dash/get_started.html"
    print(settings.DATABASES["default"])
    context = {
        "host": settings.DATABASES["default"]["HOST"] or "localhost",
        "name": settings.DATABASES["default"]["NAME"],
        "port": settings.DATABASES["default"]["PORT"] or 5432,
        "user": settings.DATABASES["default"]["USER"] or None,
    }
    return render(request, template, context)


def job_list(request):
    template = "dash/job_list.html"
    qs = Job.objects.all()
    show_system = "system" in request.GET
    if not show_system:
        qs = qs.exclude(load__fun="saltutil.find_job")
    context = {"job_list": qs, "has_system_jobs": show_system}
    context.update(paginate_queryset(qs, request))
    return render(request, template, context)


def job_detail(request, jid, success=None):
    template = "dash/job_detail.html"
    job = get_object_or_404(Job, jid=jid)
    filters = {"jid": jid}
    if success is not None:
        filters["success"] = success
    qs = Result.objects.filter(**filters)
    count = qs.count()
    if count == 1:
        return HttpResponseRedirect(
            reverse(
                "dash:result_detail", kwargs={"jid": qs[0].jid, "minion": qs[0].minion}
            )
        )
    context = {"job": job, "result_list": qs}
    context.update(paginate_queryset(qs, request))
    return render(request, template, context)


def result_list(request):
    template = "dash/result_list.html"
    qs = Result.objects.all()
    filters = {}
    show_system = "system" in request.GET
    if not show_system:
        qs = qs.exclude(fun="saltutil.find_job")
    if "minion" in request.GET:
        filters["minion"] = request.GET["minion"]
    qs = qs.filter(**filters)
    context = {"result_list": qs, "filters": filters, "has_system_jobs": show_system}
    context.update(paginate_queryset(qs, request))
    return render(request, template, context)


def result_detail(request, jid, minion):
    template = "dash/result_detail.html"
    result = get_object_or_404(Result, jid=jid, minion=minion)
    context = {"job": result.job, "result": result}
    return render(request, template, context)



# Generated by Django 2.0.1 on 2018-01-16 21:47

import django.contrib.postgres.fields
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Job',
            fields=[
                ('jid', models.CharField(max_length=50, primary_key=True, serialize=False, verbose_name='job id')),
                ('load', django.contrib.postgres.fields.jsonb.JSONField()),
            ],
            options={
                'db_table': 'jids',
                'ordering': ['-jid'],
            },
        ),
        migrations.CreateModel(
            name='Result',
            fields=[
                ('auto_id', models.AutoField(primary_key=True, serialize=False)),
                ('minion', models.CharField(db_column='id', db_index=True, max_length=100)),
                ('jid', models.CharField(db_index=True, max_length=50, verbose_name='job id')),
                ('fun', models.CharField(max_length=100, verbose_name='function')),
                ('fun_args', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None, verbose_name='function args')),
                ('completed', models.DateTimeField(db_column='alter_time')),
                ('full_ret', django.contrib.postgres.fields.jsonb.JSONField(verbose_name='full result')),
                ('return_val', django.contrib.postgres.fields.jsonb.JSONField(db_column='return', verbose_name='result')),
                ('success', models.BooleanField()),
            ],
            options={
                'db_table': 'salt_returns',
                'ordering': ['-completed'],
            },
        ),
    ]



from django.core.management import BaseCommand
from saltdash.core import server


class Command(BaseCommand):
    help = "Start Saltdash webserver"

    def handle(self, *args, **options):
        server.start()



from django.test import TestCase, override_settings
from model_mommy import mommy
from saltdash.dash.models import Job, Result


class ModelTestCase(TestCase):
    def test_list_targets_not_list(self):
        # lxc.list returns non-list results of type 'list' :(
        job = Job(load={"tgt": "minion", "tgt_type": "list"})
        self.assertListEqual(job.targets, ["minion"])

    def test_list_targets(self):
        job = Job(load={"tgt": ["minion1", "minion2"], "tgt_type": "list"})
        self.assertListEqual(job.targets, ["minion1", "minion2"])

    def test_glob_targets(self):
        job = Job(load={"tgt": "minion*", "tgt_type": "glob"})
        self.assertListEqual(job.targets, ["minion*"])

    @override_settings(HIDE_OUTPUT=["pillar.*"])
    def test_hidden_output(self):
        result = mommy.prepare(Result, full_ret={"fun": "pillar.get"})
        self.assertEqual(result.result_type, "hidden")

        result = mommy.prepare(Result, full_ret={"fun": "state.sls"})
        self.assertNotEqual(result.result_type, "hidden")



import codecs
import logging
import os
import socket
import time
from urllib.parse import quote

import waitress

from .. import config, wsgi

log = logging.getLogger(__name__)


class NonBindingServer(waitress.server.UnixWSGIServer):
    """
    Bypass binding to the socket.
    In the case of Systemd sockets, it is already bound.
    """

    def bind_server_socket(self):
        pass


def systemd_notify(msg):
    """
    Allow app to be setup as `Type=notify` in systemd
    Sending signals when it is ready to start serving.
    """
    if "NOTIFY_SOCKET" not in os.environ:
        log.debug("No systemd NOTIFY_SOCKET set")
        return
    msg = codecs.latin_1_encode(msg)[0]
    addr = os.environ["NOTIFY_SOCKET"]
    if addr[0] == "@":
        addr = "\0" + addr[1:]
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
    try:
        sock.connect(addr)
        sock.sendall(msg)
    except:
        log.exception("Could not notify systemd")


def start():
    """Start webserver"""
    logged_app = TransLogger(wsgi.application)
    # Work with systemd socket activation
    if "LISTEN_FDNAMES" in os.environ and not config.LISTEN:
        wsgi_server = NonBindingServer(
            logged_app, _sock=socket.fromfd(3, socket.AF_UNIX, socket.SOCK_STREAM)
        )
    elif config.LISTEN.startswith("/"):
        wsgi_server = waitress.server.create_server(
            logged_app, unix_socket=config.LISTEN
        )
    elif config.LISTEN:
        wsgi_server = waitress.server.create_server(logged_app, listen=config.LISTEN)
    else:
        raise RuntimeError("No socket to listen on. `LISTEN` config value is empty.")
    if wsgi.warmup.status_code != 200:
        log.error(
            "Warmup failed with status code %s. Shutting down.", wsgi.warmup.status_code
        )
        exit(1)
    wsgi_server.print_listen("Serving on {}:{}")
    systemd_notify("READY=1")
    wsgi_server.run()


# (c) 2005 Ian Bicking and contributors; written for Paste (http://pythonpaste.org)
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
# Middleware for logging requests, using Apache combined log format
class TransLogger(object):
    """
    This logging middleware will log all requests as they go through.
    They are, by default, sent to a logger named ``'wsgi'`` at the
    INFO level.

    If ``setup_console_handler`` is true, then messages for the named
    logger will be sent to the console.
    """

    format = (
        "%(REMOTE_ADDR)s - %(REMOTE_USER)s [%(time)s] "
        '"%(REQUEST_METHOD)s %(REQUEST_URI)s %(HTTP_VERSION)s" '
        '%(status)s %(bytes)s "%(HTTP_REFERER)s" "%(HTTP_USER_AGENT)s"'
    )

    def __init__(
        self,
        application,
        logger=None,
        format=None,
        logging_level=logging.INFO,
        logger_name="wsgi",
        setup_console_handler=True,
        set_logger_level=logging.DEBUG,
    ):
        if format is not None:
            self.format = format
        self.application = application
        self.logging_level = logging_level
        self.logger_name = logger_name
        if logger is None:
            self.logger = logging.getLogger(self.logger_name)
            if setup_console_handler:
                console = logging.StreamHandler()
                console.setLevel(logging.DEBUG)
                # We need to control the exact format:
                console.setFormatter(logging.Formatter("%(message)s"))
                self.logger.addHandler(console)
                self.logger.propagate = False
            if set_logger_level is not None:
                self.logger.setLevel(set_logger_level)
        else:
            self.logger = logger

    def __call__(self, environ, start_response):
        start = time.localtime()
        req_uri = quote(environ.get("SCRIPT_NAME", "") + environ.get("PATH_INFO", ""))
        if environ.get("QUERY_STRING"):
            req_uri += "?" + environ["QUERY_STRING"]
        method = environ["REQUEST_METHOD"]

        def replacement_start_response(status, headers, exc_info=None):
            # @@: Ideally we would count the bytes going by if no
            # content-length header was provided; but that does add
            # some overhead, so at least for now we'll be lazy.
            bytes = None
            for name, value in headers:
                if name.lower() == "content-length":
                    bytes = value
            self.write_log(environ, method, req_uri, start, status, bytes)
            return start_response(status, headers)

        return self.application(environ, replacement_start_response)

    def write_log(self, environ, method, req_uri, start, status, bytes):
        if bytes is None:
            bytes = "-"
        if time.daylight:
            offset = time.altzone / 60 / 60 * -100
        else:
            offset = time.timezone / 60 / 60 * -100
        if offset >= 0:
            offset = "+%0.4d" % (offset)
        elif offset < 0:
            offset = "%0.4d" % (offset)
        remote_addr = "-"
        if environ.get("HTTP_X_FORWARDED_FOR"):
            remote_addr = environ["HTTP_X_FORWARDED_FOR"]
        elif environ.get("REMOTE_ADDR"):
            remote_addr = environ["REMOTE_ADDR"]
        d = {
            "REMOTE_ADDR": remote_addr,
            "REMOTE_USER": environ.get("REMOTE_USER") or "-",
            "REQUEST_METHOD": method,
            "REQUEST_URI": req_uri,
            "HTTP_VERSION": environ.get("SERVER_PROTOCOL"),
            "time": time.strftime("%d/%b/%Y:%H:%M:%S ", start) + offset,
            "status": status.split(None, 1)[0],
            "bytes": bytes,
            "HTTP_REFERER": environ.get("HTTP_REFERER", "-"),
            "HTTP_USER_AGENT": environ.get("HTTP_USER_AGENT", "-"),
        }
        message = self.format % d
        self.logger.log(self.logging_level, message)



from unittest import TestCase

from model_mommy import mommy

from ..models import Result, _convert_state


class StateTestCase(TestCase):
    def test_requisite_state(self):
        key = "service_|-saltdash_uwsgi_|-saltdash_uwsgi_|-running"
        state_raw = {
            "result": False,
            "__sls__": "projects.saltdash",
            "changes": {},
            "comment": (
                "One or more requisite failed: "
                "projects.saltdash.update saltdash .env, "
                "projects.saltdash.saltdash_ready"
            ),
            "__run_num__": 226,
        }
        state = _convert_state(key, state_raw)
        self.assertEqual(state["status"], "requisite-failed")
        self.assertFalse(state["success"])

    def test_counts(self):
        states_raw = {
            "service_|-saltdash_uwsgi_|-saltdash_uwsgi_|-running": {
                "result": False,
                "__sls__": "projects.saltdash",
                "changes": {},
                "comment": (
                    "One or more requisite failed: "
                    "projects.saltdash.update saltdash .env, "
                    "projects.saltdash.saltdash_ready"
                ),
                "__run_num__": 226,
            }
        }
        result = mommy.prepare(Result, return_val=states_raw)
        self.assertEqual(len(result.states), 1)
        self.assertEqual(result.states_failed_requisite, 1)
        self.assertEqual(result.states_failed, 0)

    def test_state_ordering(self):
        states_raw = {
            "pkg_|-graphite_api_install_|-python-dev_|-installed": {
                "name": "python-dev",
                "__id__": "graphite_api_install",
                "result": True,
                "__sls__": "packages.graphite.api",
                "changes": {},
                "comment": "All specified packages are already installed",
                "duration": 12.633,
                "start_time": "07:20:17.316386",
                "__run_num__": 167,
            },
            "service_|-saltdash_uwsgi_|-saltdash_uwsgi_|-running": {
                "result": False,
                "__sls__": "projects.saltdash",
                "changes": {},
                "comment": "One or more requisite failed: projects.saltdash.update saltdash .env, projects.saltdash.saltdash_ready",
                "__run_num__": 226,
            },
            "file_|-/var/lib/sentry_|-/var/lib/sentry_|-directory": {
                "name": "/var/lib/sentry",
                "__id__": "/var/lib/sentry",
                "result": True,
                "__sls__": "docker.sentry",
                "changes": {},
                "comment": "Directory /var/lib/sentry is in the correct state\nDirectory /var/lib/sentry updated",
                "duration": 1.473,
                "pchanges": {},
                "start_time": "07:20:07.085770",
                "__run_num__": 137,
            },
            "file_|-/var/log/backup_|-/var/log/backup_|-directory": {
                "name": "/var/log/backup",
                "__id__": "/var/log/backup",
                "result": True,
                "__sls__": "lincolnloop.backup_to_s3",
                "changes": {},
                "comment": "Directory /var/log/backup is in the correct state\nDirectory /var/log/backup updated",
                "duration": 2.237,
                "pchanges": {},
                "start_time": "07:19:53.316698",
                "__run_num__": 81,
            },
        }
        result = mommy.prepare(Result, return_val=states_raw)
        self.assertListEqual([s["order"] for s in result.states], [81, 137, 167, 226])
